/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is this file as it was released on
 * January 3, 2001.
 * 
 * The Initial Developer of the Original Code is Jonas Sicking.
 * Portions created by Jonas Sicking are Copyright (C) 2000
 * Jonas Sicking.  All Rights Reserved.
 * 
 * Contributor(s):
 *   Jonas Sicking <sicking@bigfoot.com> (Original Author)
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 *
 */

var XLinkNS = "http://www.w3.org/1999/xlink";
var HTMLNS = "http://www.w3.org/1999/xhtml";
var XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
var bundle;
var nodeView;

function onLoad() 
{
    bundle = srGetStrBundle("chrome://navigator/locale/properties.properties");
    showPropertiesFor(window.arguments[0]);

    nodeView = window.arguments[0].ownerDocument.defaultView;
}

function showPropertiesFor(elem)
{
    while (elem && elem.nodeType != 1)
        elem = elem.parentNode;

    if (!elem) {
        alert(bundle.GetStringFromName("unableToShowProps"));
        window.close();
    }

    showDocInfo(elem);
    showLinkInfo(elem);
    showImageInfo(elem);
    showInsDelInfo(elem);
    showQuoteInfo(elem);
    showMiscInfo(elem);
}


function showDocInfo(elem)
{
    setInfo("document-url", elem.ownerDocument.location.href);

    if(elem.ownerDocument.title)
        setInfo("document-title", elem.ownerDocument.title);
    else
        setInfo("document-title", "");
        
}


function showImageInfo(elem)
{
    var img;
    var imgType;   // "img" = <img>
                   // "object" = <object>
                   // "input" = <input type=image>
                   // "background" = css background (to be added later)

    if (elem.tagName.toUpperCase() == "IMG") {
        img = elem;
        imgType = "img";

    } else if (elem.tagName.toUpperCase() == "OBJECT" &&
               elem.getAttribute("type").substring(0,6) == "image/" &&
               elem.getAttribute("data") != "") {
        img = elem;
        imgType = "object";
        
    } else if (elem.tagName.toUpperCase() == "INPUT" &&
               elem.getAttribute("type").toUpperCase() == "IMAGE") {
        img = elem;
        imgType = "input";
        
    } else if (elem.tagName.toUpperCase() == "AREA" || 
               elem.tagName.toUpperCase() == "A" ) {
        
        // Clicked in image map?
        var map = elem;
        while (map && map.nodeType == 1 && map.tagName.toUpperCase() != "MAP" )
            map = map.parentNode;
        
        if (map && map.nodeType == 1) {
            // Find <img> or <object> which uses found imagemap.
            // If more then one object is found we can't determin which one
            // was clicked.
            
            var mapuri = "#" + map.getAttribute("name");
            var multipleFound = false;
            
            var list = elem.ownerDocument.getElementsByTagName("IMG");
            for(var i=0; i < list.length; i++) {
                if (list.item(i).getAttribute("usemap") == mapuri) {
                    if (img) {
                        multipleFound = true;
                        break;
                    } else {
                        img = list.item(i);
                        imgType = "img";
                    }
                }
            }
            
            var list = elem.ownerDocument.getElementsByTagName("OBJECT");
            for(var i=0; i < list.length; i++) {
                if (list.item(i).getAttribute("usemap") == mapuri) {
                    if (img) {
                        multipleFound = true;
                        break;
                    } else {
                        img = list.item(i);
                        imgType = "object";
                    }
                }
            }
            
            if (multipleFound)
                img = false;
        }
    }

    if (img) {
        setInfo("image-url", img.src);
        if (imgType == "img") {
            setInfo("image-desc", getAbsoluteURL(img.getAttribute("longdesc"), img));
        } else {
            setInfo("image-desc", "");
        }
    }
    else
        hideNode("sec-image");
}

function showLinkInfo(elem)
{
    var node = elem;
    var linkFound = false;
    
    while (node && node.nodeType == 1 && !linkFound) {
        if ((node.tagName.toUpperCase() == "A" &&
             node.getAttribute("href") != "") ||
             node.tagName.toUpperCase() == "AREA" ) {
            
            linkFound = true;
            setInfo("link-url", node.href);
            setInfo("link-lang", node.getAttribute("hreflang"));
            setInfo("link-type", node.getAttribute("type"));
            setInfo("link-rel", node.getAttribute("rel"));
            setInfo("link-rev", node.getAttribute("rev"));
            
            target = node.getAttribute("target");
            if ( target == "") {
                var baseTags = node.ownerDocument.getElementsByTagName("base");
                for (var i=0; baseTags && i<baseTags.length; i++) {
                    if (baseTags.item(i).hasAttribute("target")) {
                        target = baseTags.item(i).getAttribute("target");
                    }
                }
            }
            
            switch( target ) {
            case "_top":
                setInfo("link-target", bundle.GetStringFromName("sameWindowText"));
                break;
            case "_parent":
                setInfo("link-target", bundle.GetStringFromName("parentFrameText"));
                break;
            case "_blank":
                setInfo("link-target", bundle.GetStringFromName("newWindowText"));
                break;
            case "":
            case "_self":
                if ( node.ownerDocument != node.ownerDocument.defaultView._content.document )
                    setInfo("link-target", bundle.GetStringFromName("sameFrameText"));
                else
                    setInfo("link-target", bundle.GetStringFromName("sameWindowText"));
                break;
            default:
                setInfo("link-target", "\"" + target + "\"");
            }
        }

        else if (node.getAttributeNS(XLinkNS,"href") != "") {
            linkFound = true;
            setInfo("link-url", getAbsoluteURL(node.getAttributeNS(XLinkNS,"href"),node));
            setInfo("link-lang", "");
            setInfo("link-type", "");
            setInfo("link-rel", "");
            setInfo("link-rev", "");

            switch( node.getAttributeNS(XLinkNS,"show") ) {
            case "embed":
                setInfo("link-target", bundle.GetStringFromName("embeddedText"));
                break;
            case "new":
                setInfo("link-target", bundle.GetStringFromName("newWindowText"));
                break;
            case "":
            case "replace":
                if ( node.ownerDocument != node.ownerDocument.defaultView._content.document )
                    setInfo("link-target", bundle.GetStringFromName("sameFrameText"));
                else
                    setInfo("link-target", bundle.GetStringFromName("sameWindowText"));
                break;
            default:
                setInfo("link-target", "");
                break;
            }
        }
        node = node.parentNode;
    }

    if (!linkFound)
        hideNode("sec-link");

}

function showInsDelInfo(elem)
{
    var node = elem;

    while (node && node.nodeType == 1 &&
           node.tagName.toUpperCase() != "INS" &&
           node.tagName.toUpperCase() != "DEL") {
        node = node.parentNode;
    }

    if (node && node.nodeType == 1 &&
        (node.getAttribute("cite") != "" ||
         node.getAttribute("datetime") != "")) {
        setInfo("insdel-cite", getAbsoluteURL(node.getAttribute("cite"),node));
        setInfo("insdel-date", node.getAttribute("datetime"));
    } else {
        hideNode("sec-insdel");
    }
}


function showQuoteInfo(elem)
{
    var node = elem;

    while (node && node.nodeType == 1 &&
           node.tagName.toUpperCase() != "Q" &&
           node.tagName.toUpperCase() != "BLOCKQUOTE") {
        node = node.parentNode;
    }

    if (node && node.nodeType == 1) {
        setInfo("quote-cite", getAbsoluteURL(node.getAttribute("cite"),node));
    } else {
        hideNode("sec-quote");
    }
}


function showMiscInfo(elem)
{
    var miscFound = false;

    // language
    var node = elem;
    while (node && node.nodeType == 1 &&
           node.getAttribute("lang") == "" &&
           node.getAttribute("xml:lang") == "") {
        node = node.parentNode;
    }

    if (node && node.nodeType == 1) {
        miscFound = true;
        setInfo("misc-lang", node.getAttribute("lang") || node.getAttribute("xml:lang"));
    } else {
        setInfo("misc-lang", "");
    }

    // title (same as tooltip)
    var node = elem;
    while (node && node.nodeType == 1 &&
           node.getAttribute("title") == "") {
        node = node.parentNode;
    }

    if (node && node.nodeType == 1) {
        miscFound = true;
        setInfo("misc-title", node.getAttribute("title"));
    } else {
        setInfo("misc-title", "");
    }

    // table summary
    // tables could be nested so we walk up until we find one with summary
    var node = elem;
    while (node && node.nodeType == 1 &&
           (node.tagName.toUpperCase() != "TABLE" || 
            node.getAttribute("summary") == "")) {
        node = node.parentNode;
    }

    if (node && node.nodeType == 1) {
        miscFound = true;
        setInfo("misc-tblsummary", node.getAttribute("summary"));
    } else {
        setInfo("misc-tblsummary", "");
    }

    if (!miscFound) {
        hideNode("sec-misc");
    }
}


// Set text of node id to value
// if value="" the node with specified id is hidden.
// Node should be have one of theese forms
// <xul:text id="id-text" value=""/>
// <xul:html id="id-text"/>
function setInfo(id, value)
{
    if (value == "") {
        hideNode(id);
        return;
    }
    
    var node = document.getElementById(id+"-text");

    if (node.namespaceURI == XULNS && node.localName == "text") {
        node.setAttribute("value",value);

    } else if (node.namespaceURI == XULNS && node.localName == "html") {
        while (node.hasChildNodes())
            node.removeChild(node.firstChild);
        node.appendChild(node.ownerDocument.createTextNode(value));

    }

}

// Hide node with specified id
function hideNode(id)
{
    var style = document.getElementById(id).getAttribute("style");
    document.getElementById(id).setAttribute("style", "display:none;" + style);
}

// Needs to take care of XMLBase and <base>
// url is the possibly relative url.
// node is the node where the url was given (neede for XMLBase)
// if "" is given as url empty string is returned
function getAbsoluteURL(url, elem)
{
    if(url == "")
        return "";

    var URL = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces["nsIURL"]);

    var urlArr = new Array(url);
    var node = elem;

    while (node && node.nodeType == 1) {
        if(node.getAttribute("xml:base") != "")
            urlArr.unshift(node.getAttribute("xml:base"));

        node = node.parentNode;
    }
    
    // Look for a <base>.
    var baseTags = elem.ownerDocument.getElementsByTagName("base");
    if ( baseTags && baseTags.length ) {
        urlArr.unshift(baseTags[baseTags.length - 1].getAttribute("href"));
    }

    // relsolve everything from bottom up, staring with document location
    URL.spec = elem.ownerDocument.location.href;
    for(var i=0; i<urlArr.length; i++) {
        URL.spec = URL.resolve(urlArr[i]);
    }
    

    return URL.spec;
}

//opens the link containd in the node's "value" attribute.
function openLink(node)
{
    var url = node.getAttribute("value");
    nodeView._content.document.location = url;
}

